/* AsmCodeWriter.java */

package whitespace;

import java.io.*;
import java.util.*;

import whitespace.Instruction;
import whitespace.InstructionNode;

public class AsmCodeWriter {
  
  private List<InstructionNode> list;
  private PrintStream out;

  public AsmCodeWriter(List<InstructionNode> list, OutputStream out) {
    this.list = list;
    this.out = new PrintStream(out);
  }

  public void writeCode() throws IOException {
    for (InstructionNode node : list) {
      if (node.opcode == Instruction.Label) {
	String label = node.labelParam;
	out.printf("%s:", label);out.println();
      } else {
	InstructionInfo info = 
	  Instruction.getInstructionInfoByOpcode(node.opcode);
	if (info.arity == Instruction.NO_PARAMS) {
	  out.printf("\t%s", info.name);out.println();
	} else if (info.arity == Instruction.LABEL_PARAM) {
	  out.printf("\t%s\t%s", info.name, node.labelParam);out.println();
	} else if (info.arity == Instruction.NUMBER_PARAM) {
	  out.printf("\t%s\t%d", info.name, node.numberParam);out.println();
	} else {
	  assert false : "Illegal instruction artity";
	}
      }
    }
    out.flush();
  }
}

	
