/* Instruction.java */

package whitespace;

import whitespace.InstructionInfo;

public class Instruction {
  public static final int Push       = 0;
  public static final int Dup        = 1;
  public static final int Swap       = 2;
  public static final int Discard    = 3;
  public static final int Add        = 4;
  public static final int Mul        = 5;
  public static final int Sub        = 6;
  public static final int Div        = 7;
  public static final int Mod        = 8;
  public static final int Store      = 9;
  public static final int Retrieve   = 10;
  public static final int Label      = 11;
  public static final int Call       = 12;
  public static final int Jump       = 13;
  public static final int JumpIfZero = 14;
  public static final int JumpIfNeg  = 15;
  public static final int Ret        = 16;
  public static final int Exit       = 17;
  public static final int OutChar    = 18;
  public static final int OutNum     = 19;
  public static final int ReadChar   = 20;
  public static final int ReadNum    = 21;
  public static final int NUMBER_OF_INSTRUCTIONS = 22;

  public static final int NOT_A_OPCODE = 255;
  public static final int MAX_OF_NUMBER_BITS = 56;
  public static final int MAX_OF_LABEL_BITS = 128;

  public static final int NO_PARAMS = 0;
  public static final int NUMBER_PARAM = 1;
  public static final int LABEL_PARAM = 2;

  private static final InstructionInfo[]  info;
  static {
    info = new InstructionInfo[NUMBER_OF_INSTRUCTIONS];
    info[Push] = new InstructionInfo(Push,
				     "Push",
				     NUMBER_PARAM,
				     "Push the number onto the stack"
				     );
    info[Dup] = new InstructionInfo(Dup,
				     "Dup",
				     NO_PARAMS,
				     "Duplicate the top item on the stack"
				     );
    info[Swap] = new InstructionInfo(Swap,
				     "Swap",
				     NO_PARAMS,
				     "Swap th top two items on the stack"
				     );
    info[Discard] = new InstructionInfo(Discard,
				     "Discard",
				     NO_PARAMS,
				     "Discard the top item on the stack"
				     );
    info[Add] = new InstructionInfo(Add,
				     "Add",
				     NO_PARAMS,
				     "Addition"
				     );
    info[Mul] = new InstructionInfo(Mul,
				     "Mul",
				     NO_PARAMS,
				     "Multiplication"
				     );
    info[Sub] = new InstructionInfo(Sub,
				     "Sub",
				     NO_PARAMS,
				     "Subtraction"
				     );
    info[Div] = new InstructionInfo(Div,
				     "Div",
				     NO_PARAMS,
				     "Integer Division"
				     );
    info[Mod] = new InstructionInfo(Mod,
				     "Mod",
				     NO_PARAMS,
				     "Modulo"
				     );
    info[Store] = new InstructionInfo(Store,
				     "Store",
				     NO_PARAMS,
				     "Store "
				     );
    info[Retrieve] = new InstructionInfo(Retrieve,
				     "Retrieve",
				     NO_PARAMS,
				     "Retrieve "
				     );
    info[Call] = new InstructionInfo(Call,
				     "Call",
				     LABEL_PARAM,
				     "Call a subroutine"
				     );
    info[Jump] = new InstructionInfo(Jump,
				     "Jump",
				     LABEL_PARAM,
				     "Jump unconditionally to a label"
				     );
    info[JumpIfZero] = new InstructionInfo(JumpIfZero,
				     "JumpIfZero",
				     LABEL_PARAM,
				     "Jump to a label if the top of the stack is zero"
				     );
    info[JumpIfNeg] = new InstructionInfo(JumpIfNeg,
				     "JumpIfNeg",
				     LABEL_PARAM,
				     "Jump to a label if the top of the stack is negative"
				     );
    info[Ret] = new InstructionInfo(Ret,
				     "Ret",
				     NO_PARAMS,
				     "End a subroutine and transfer control back to the caller"
				     );
    info[Exit] = new InstructionInfo(Exit,
				     "Exit",
				     NO_PARAMS,
				     "End the program"
				     );
    info[OutChar] = new InstructionInfo(OutChar,
				     "OutChar",
				     NO_PARAMS,
				     "Output the character at the top of the stack"
				     );
    info[OutNum] = new InstructionInfo(OutNum,
				     "OutNum",
				     NO_PARAMS,
				     "Output the number at the top of the stack"
				     );
    info[ReadChar] = new InstructionInfo(ReadChar,
				     "ReadChar",
				     NO_PARAMS,
				     "Read a character and place it in the location given by the top of the stack"
				     );
    info[ReadNum] = new InstructionInfo(ReadNum,
				     "ReadNum",
				     NO_PARAMS,
				     "Read a number and place it in the location given by the top of the stack"
				     );

  } // end of static block


  public static InstructionInfo getInstructionInfoByOpcode(int opcode) {
    if (opcode < 0 || info.length <= opcode) {
      return null;
    }
    return info[opcode];
  }

  public static InstructionInfo getInstructionInfoByName(String name) {
    for (int i = 0; i < info.length; i++) {
      if (info[i].name.equals(name)) {
	return info[i];
      }
    }
    return null;
  }
}
