/* Interp.java */

package whitespace;

import java.io.*;
import java.util.*;

import whitespace.IOEnvironment;
import whitespace.InterpException;
import whitespace.Instruction;
import whitespace.InstructionNode;
import whitespace.Inspector;

// ѐԒňĂȂ
// ѐe[uɂȂƂ́H

public class Interp {
  private static final int HEAP_SIZE = 16 * 1024;
  private static final long MAX_LOOP_COUNT = 10 * (1000 * 10000); //10

  private Stack<Long> dataStack;
  private Stack<Integer> callStack;
  private Map<String, Integer> jumpTable;
  private long[] heap;
  private List<InstructionNode> program;
  private IOEnvironment env;

  public Interp(List<InstructionNode> program, IOEnvironment env) {
    dataStack = new Stack<Long>();
    callStack = new Stack<Integer>();
    jumpTable = new HashMap<String, Integer>();
    heap = new long[HEAP_SIZE];
    this.program = program;
    this.env = env;
  }

  private Inspector inspector = null;

  public void setInspector(Inspector inspector) {
    this.inspector = inspector;
  }

  private boolean step = false;
  public void setStepping(boolean val) throws InterpException {
    if (val == true && inspector == null) {
      throw new InterpException("Inspector is NOT installed");
    }
    step = val;
  }

  public boolean getStepping() {
    return step;
  }

  private void resolvLabels() {
    int pos = 0;
    for (InstructionNode node : program) {
      if (node.opcode == Instruction.Label) {
	jumpTable.put(node.labelParam, pos + 1);
      }
      pos++;
    }
/*    for (String key : jumpTable.keySet()) {
      System.out.println(key + "==>" + jumpTable.get(key));
    }
*/
  }

  public void execute() throws InterpException, IOException {
    resolvLabels();
    int pc = 0;
    boolean going = true;
    long count;
    for (count = 0L; going && count < MAX_LOOP_COUNT; count++) {
      long val;
      InstructionNode node = program.get(pc);
      if (step) {
	if (inspector.inspect(pc, program, dataStack, callStack) 
	    == Inspector.BREAK) {
	  inspector.finishInspection();
	  break;
	}
	// continue
      }
      switch (node.opcode) {
      case Instruction.Push:
	pc++;
	doPush(node.numberParam);
	break;
      case Instruction.Dup:
	pc++;
	doDup();
	break;
      case Instruction.Swap:
	pc++;
	doSwap();
	break;
      case Instruction.Discard:
	pc++;
	doDiscard();
	break;
      case Instruction.Add:
	pc++;
	doAdd();
	break;
      case Instruction.Mul:
	pc++;
	doMul();
	break;
      case Instruction.Sub:
	pc++;
	doSub();
	break;
      case Instruction.Div:
	pc++;
	doDiv();
	break;
      case Instruction.Mod:
	pc++;
	doMod();
	break;
      case Instruction.Store:
	pc++;
	doStore();
	break;
      case Instruction.Retrieve :
	pc++;
	doRetrieve();
	break;
      case Instruction.Label :
	pc++;
	// skip
	break;
      case Instruction.Call :
	callStack.push(pc + 1); // ߂Ԓn̕ۑ
	pc = jumpTable.get(node.labelParam);
	break;
      case Instruction.Jump :
	pc = jumpTable.get(node.labelParam);
	break;
      case Instruction.JumpIfZero:
	val = dataStack.pop();
	if (val == 0L) {
	  pc = jumpTable.get(node.labelParam);
	} else {
	  pc++;
	}
	break;
      case Instruction.JumpIfNeg:
	val = dataStack.pop();
	if (val < 0L) {
	  pc = jumpTable.get(node.labelParam);
	} else {
	  pc++;
	}
	break;
      case Instruction.Ret:
	pc = callStack.pop();
	break;
      case Instruction.Exit:
	going = false;
	break;
      case Instruction.OutChar:
	pc++;
	doOutChar();
	break;
      case Instruction.OutNum:
	pc++;
	doOutNum();
	break;
      case Instruction.ReadChar:
	pc++;
	doReadChar();
	break;
      case Instruction.ReadNum:
	pc++;
	doReadNum();
	break;
      default:
	assert false : "Illegal opcode";
      }
    }
    if (step) inspector.finishInspection();
    if (count >= MAX_LOOP_COUNT) {
      throw new InterpException("too many loop count");
    }
  }

/* BEGIN: automatically generated code */
    private void doPush(long val) {
      dataStack.push(val);
    }
    private void doDup() {
      long tmp = dataStack.peek();
      dataStack.push(tmp);
    }
    private void doSwap() {
      long tmp1 = dataStack.pop();
      long tmp2 = dataStack.pop();
      dataStack.push(tmp1);
      dataStack.push(tmp2);
    }
    private void doDiscard() {
      dataStack.pop();
    }
    private void doAdd() {
      long arg2 = dataStack.pop();
      long arg1 = dataStack.pop();
      dataStack.push(arg1 + arg2);
    }
    private void doMul() {
      long arg2 = dataStack.pop();
      long arg1 = dataStack.pop();
      dataStack.push(arg1 * arg2);

    }
    private void doSub() {
      long arg2 = dataStack.pop();
      long arg1 = dataStack.pop();
      dataStack.push(arg1 - arg2);
    }
    private void doDiv() {
      long arg2 = dataStack.pop();
      long arg1 = dataStack.pop();
      dataStack.push(arg1 / arg2);
    }
    private void doMod() {
      long arg2 = dataStack.pop();
      long arg1 = dataStack.pop();
      dataStack.push(arg1 % arg2);
    }
    private void doStore() {
      long val = dataStack.pop();
      long addr = dataStack.pop();
      int i = (int)addr;
      heap[i] = val;
    }
    private void doRetrieve() {
      long addr = dataStack.pop();
      int i = (int)addr;
      dataStack.push(heap[i]);
    }

    private void doOutChar() throws IOException {
      long tmp = dataStack.pop();
      char c = (char)tmp;
      env.outChar(c);
    }
    private void doOutNum() throws IOException {
      long num = dataStack.pop();
      env.outNum(num);
    }
    private void doReadChar() throws IOException {
      long tmp = dataStack.pop();
      int addr = (int)tmp;
      long val = (long)env.readChar();
      heap[addr] = val;
    }
    private void doReadNum() throws IOException {
      long tmp = dataStack.pop();
      int addr = (int)tmp;
      long val = env.readNum();
      heap[addr] = val;
    }
/* END: automatically generated code */

}


