/* Lexer.java */
package whitespace;

import java.io.*;
import whitespace.Token;
import whitespace.WhitespaceChars;

public class Lexer {

  private final int SPACE;
  private final int TAB;
  private final int LF;
  private final int CR;
  private final int NUL;

  private PushbackInputStream input = null;
  private String sourceName = null;
  private int sourceLineNo = -1; // not supported
  private int sourceCol = -1; // not supported

  public Lexer(InputStream in, String name, WhitespaceChars chars) {
    this.input = new PushbackInputStream(in);
    this.sourceName = name;
    if (chars == null) {
      this.SPACE = 32;
      this.TAB = 9;
      this.LF = 10;
      this.CR = 13;
      this.NUL = 0;
    } else {
      this.SPACE = chars.SPACE;
      this.TAB = chars.TAB;
      this.LF = chars.LF;
      this.CR = chars.CR;
      this.NUL = chars.NUL;
    }
  }
  public Lexer(InputStream in, String name) {
    this(in, name, null);
  }
  public Lexer(InputStream in) {
    this(in, null);
  }

  private boolean eof = false; // EOFzĂ͓ǂ܂ȂƂۏ؂

  public Token getNextToken() throws IOException {
    if (eof) {
      return new  Token(Token.EOF);
    }
    int c; 
    while (true) {
      c = input.read();
      if (c == -1) {
	eof = true;
	return new Token(Token.EOF);
      }
      if (c == SPACE) {
	return new Token(Token.Space);
      }
      if (c == TAB) {
	return new Token(Token.Tab);
      }
      if (c == LF) {
	return new Token(Token.LF);
      }
      // CR̓ꏈ
      if (c == CR) {
	// NULǂݔ΂
	while ((c = input.read()) != -1) {
	  if (c != NUL) break;
	}
	// CR+EOF or CR+LF ==> LF
	// CR+Other ==> pushback + LF
	if (c != -1 && c != LF) {
	  input.unread(c);
	}
	if (c == -1) {
	  eof = true;
	}
	return new Token(Token.LF);
      }
      // ̑̓[vp
    }
  }
}
   
      

