/* LogInspector.java */

package whitespace;

import java.io.*;
import java.util.*;

import whitespace.Inspector;
import whitespace.InstructionNode;
import whitespace.Instruction;

public class LogInspector implements Inspector {
  private int count;
  private PrintWriter log;
  public LogInspector() throws IOException {
    count = 0;
    log = new PrintWriter(new FileOutputStream(new File("whitespace.log")));
  }

  public int inspect(int pc, List<InstructionNode> program, 
		     Stack<Long> dataStack, Stack<Integer> callStack){
    if (count >= 1000) {
      System.err.println("LogInspector: count up!");
      log.close();
      return Inspector.BREAK;
    }
    count++;
    InstructionNode node = program.get(pc);
    InstructionInfo info = Instruction.getInstructionInfoByOpcode(node.opcode);
    String stackTop;
    if (dataStack.size() != 0) {
      stackTop = dataStack.peek().toString();
    } else {
      stackTop = "()";
    }
    
    switch (info.arity) {
    case Instruction.NUMBER_PARAM:
      log.printf("%5d:pc=%4d;top=%5s;%s %d\n", count, pc, stackTop, 
		 info.name, node.numberParam);
      break;
    case Instruction.LABEL_PARAM:
      log.printf("%5d:pc=%4d;top=%5s;%s %s\n", count, pc, stackTop, 
		 info.name, node.labelParam);
      break;
    case  Instruction.NO_PARAMS:
      log.printf("%5d:pc=%4d;top=%5s;%s\n", count, pc, stackTop, 
		 info.name);
      break;
    default:
      assert false : "Illegal arity";
    }
    return Inspector.CONTINUE;
  }

  public void finishInspection() {
    log.close();
  }
}
