/* Parser.java */

package whitespace;

import java.io.*;
import java.util.*;

import whitespace.Token;
import whitespace.Lexer;
import whitespace.Instruction;
import whitespace.InstructionNode;
import whitespace.ParseException;

public class Parser {
  private Lexer lexer;

  public Parser(Lexer lexer) {
    this.lexer = lexer;
  }
  public List<InstructionNode> parse() throws IOException, ParseException {
    boolean going = true;
    int tokenCode;
    long number;
    String label;
    InstructionNode node;
    List<InstructionNode> list = new ArrayList<InstructionNode>();
    /* =================================== */

    while (going) {


/* BEGIN: automatically generated code */
tokenCode = lexer.getNextToken().code;
switch (tokenCode) {
case Token.EOF: // [EOF]
  going = false;
  break;
case Token.Space: // [Space]
  tokenCode = lexer.getNextToken().code;
  switch (tokenCode) {
  case Token.Space: // [Space][Space]
    number = getNumberParameter();
    node = new InstructionNode(Instruction.Push, number);
    list.add(node);
    break;
  case Token.LF: // [Space][LF]
    tokenCode = lexer.getNextToken().code;
    switch (tokenCode) {
    case Token.Space: // [Space][LF][Space]
      node = new InstructionNode(Instruction.Dup);
      list.add(node);
      break;
    case Token.Tab: // [Space][LF][Tab]
      node = new InstructionNode(Instruction.Swap);
      list.add(node);
      break;
    case Token.LF: // [Space][LF][LF]
      node = new InstructionNode(Instruction.Discard);
      list.add(node);
      break;
    default:
      throw new ParseException("Illegal token combination [" + Token.codeToName(tokenCode) + "]");
    }
    break;
  default:
    throw new ParseException("Illegal token combination [" + Token.codeToName(tokenCode) + "]");
  }
  break;
case Token.LF: // [LF]
  tokenCode = lexer.getNextToken().code;
  switch (tokenCode) {
  case Token.Space: // [LF][Space]
    tokenCode = lexer.getNextToken().code;
    switch (tokenCode) {
    case Token.Space: // [LF][Space][Space]
      label = getLabelParameter();
      node = new InstructionNode(Instruction.Label, label);
      list.add(node);
      break;
    case Token.Tab: // [LF][Space][Tab]
      label = getLabelParameter();
      node = new InstructionNode(Instruction.Call, label);
      list.add(node);
      break;
    case Token.LF: // [LF][Space][LF]
      label = getLabelParameter();
      node = new InstructionNode(Instruction.Jump, label);
      list.add(node);
      break;
    default:
      throw new ParseException("Illegal token combination [" + Token.codeToName(tokenCode) + "]");
    }
    break;
  case Token.Tab: // [LF][Tab]
    tokenCode = lexer.getNextToken().code;
    switch (tokenCode) {
    case Token.Space: // [LF][Tab][Space]
      label = getLabelParameter();
      node = new InstructionNode(Instruction.JumpIfZero, label);
      list.add(node);
      break;
    case Token.Tab: // [LF][Tab][Tab]
      label = getLabelParameter();
      node = new InstructionNode(Instruction.JumpIfNeg, label);
      list.add(node);
      break;
    case Token.LF: // [LF][Tab][LF]
      node = new InstructionNode(Instruction.Ret);
      list.add(node);
      break;
    default:
      throw new ParseException("Illegal token combination [" + Token.codeToName(tokenCode) + "]");
    }
    break;
  case Token.LF: // [LF][LF]
    tokenCode = lexer.getNextToken().code;
    switch (tokenCode) {
    case Token.LF: // [LF][LF][LF]
      node = new InstructionNode(Instruction.Exit);
      list.add(node);
      break;
    default:
      throw new ParseException("Illegal token combination [" + Token.codeToName(tokenCode) + "]");
    }
    break;
  default:
    throw new ParseException("Illegal token combination [" + Token.codeToName(tokenCode) + "]");
  }
  break;
case Token.Tab: // [Tab]
  tokenCode = lexer.getNextToken().code;
  switch (tokenCode) {
  case Token.Space: // [Tab][Space]
    tokenCode = lexer.getNextToken().code;
    switch (tokenCode) {
    case Token.Space: // [Tab][Space][Space]
      tokenCode = lexer.getNextToken().code;
      switch (tokenCode) {
      case Token.Space: // [Tab][Space][Space][Space]
        node = new InstructionNode(Instruction.Add);
        list.add(node);
        break;
      case Token.Tab: // [Tab][Space][Space][Tab]
        node = new InstructionNode(Instruction.Sub);
        list.add(node);
        break;
      case Token.LF: // [Tab][Space][Space][LF]
        node = new InstructionNode(Instruction.Mul);
        list.add(node);
        break;
      default:
        throw new ParseException("Illegal token combination [" + Token.codeToName(tokenCode) + "]");
      }
      break;
    case Token.Tab: // [Tab][Space][Tab]
      tokenCode = lexer.getNextToken().code;
      switch (tokenCode) {
      case Token.Space: // [Tab][Space][Tab][Space]
        node = new InstructionNode(Instruction.Div);
        list.add(node);
        break;
      case Token.Tab: // [Tab][Space][Tab][Tab]
        node = new InstructionNode(Instruction.Mod);
        list.add(node);
        break;
      default:
        throw new ParseException("Illegal token combination [" + Token.codeToName(tokenCode) + "]");
      }
      break;
    default:
      throw new ParseException("Illegal token combination [" + Token.codeToName(tokenCode) + "]");
    }
    break;
  case Token.Tab: // [Tab][Tab]
    tokenCode = lexer.getNextToken().code;
    switch (tokenCode) {
    case Token.Space: // [Tab][Tab][Space]
      node = new InstructionNode(Instruction.Store);
      list.add(node);
      break;
    case Token.Tab: // [Tab][Tab][Tab]
      node = new InstructionNode(Instruction.Retrieve);
      list.add(node);
      break;
    default:
      throw new ParseException("Illegal token combination [" + Token.codeToName(tokenCode) + "]");
    }
    break;
  case Token.LF: // [Tab][LF]
    tokenCode = lexer.getNextToken().code;
    switch (tokenCode) {
    case Token.Space: // [Tab][LF][Space]
      tokenCode = lexer.getNextToken().code;
      switch (tokenCode) {
      case Token.Space: // [Tab][LF][Space][Space]
        node = new InstructionNode(Instruction.OutChar);
        list.add(node);
        break;
      case Token.Tab: // [Tab][LF][Space][Tab]
        node = new InstructionNode(Instruction.OutNum);
        list.add(node);
        break;
      default:
        throw new ParseException("Illegal token combination [" + Token.codeToName(tokenCode) + "]");
      }
      break;
    case Token.Tab: // [Tab][LF][Tab]
      tokenCode = lexer.getNextToken().code;
      switch (tokenCode) {
      case Token.Space: // [Tab][LF][Tab][Space]
        node = new InstructionNode(Instruction.ReadChar);
        list.add(node);
        break;
      case Token.Tab: // [Tab][LF][Tab][Tab]
        node = new InstructionNode(Instruction.ReadNum);
        list.add(node);
        break;
      default:
        throw new ParseException("Illegal token combination [" + Token.codeToName(tokenCode) + "]");
      }
      break;
    default:
      throw new ParseException("Illegal token combination [" + Token.codeToName(tokenCode) + "]");
    }
    break;
  default:
    throw new ParseException("Illegal token combination [" + Token.codeToName(tokenCode) + "]");
  }
  break;
default:
  assert false : "Undefined Token";
}
/* END: automatically generated code */


}


    /* =================================== */
    return list;
  }

  long getNumberParameter() throws ParseException, IOException {
    int tokenCode;
    tokenCode = lexer.getNextToken().code;
    int sign = (tokenCode == Token.Space ? 1 : (tokenCode == Token.Tab ? -1 : 0));
    if (sign == 0) {
      throw new ParseException("Illegal number sign");
    }
    long val = 0;
    int i;
    for (i = 0; i < Instruction.MAX_OF_NUMBER_BITS; i++) {
      tokenCode = lexer.getNextToken().code;
      if (tokenCode == Token.LF) break;
      if (tokenCode == Token.Tab) {
	val *= 2;
	val += 1;
      } else if (tokenCode == Token.Space) {
	val *= 2;
      } else {
	throw new ParseException("Illegal number format");
      }
    }
    if (i == Instruction.MAX_OF_NUMBER_BITS) {
      throw new ParseException("too long number");
    }
    val = sign * val;
//    debugNumber(val);
    return val;
  }
    
  String getLabelParameter() throws IOException, ParseException {
    int tokenCode;
    StringBuffer buffer = new StringBuffer();

    int i;
    int val = 0;
    for (i = 0; i < Instruction.MAX_OF_LABEL_BITS; i++) {
      if (i % 4 == 3) {
	buffer.append((char)(val + 'a'));
	val = 0;
      }
      tokenCode = lexer.getNextToken().code;
      if (tokenCode == Token.LF) break;
      if (tokenCode == Token.Tab) {
	val *= 2;
	val += 1;
      } else if (tokenCode == Token.Space) {
	val *= 2;
      } else {
	throw new ParseException("Illegal number format");
      }
    }
//    debugLabel(buffer.toString());
    int found = -1;
    if (i == Instruction.MAX_OF_LABEL_BITS) {
      for (i = 0; i < 10000; i++) {
	tokenCode = lexer.getNextToken().code;
	if (tokenCode == Token.LF) {
	  found = i;
	  break;
	} else if (tokenCode == Token.EOF) {
	  found = -1;
	  break;
	}
      }
      throw new ParseException("too long label (cannot find LF within " 
			       + Instruction.MAX_OF_LABEL_BITS
			       + " width) FOUND after "
			       + found
			       );
    }
    return buffer.toString();
  }

  private void debugNumber(long val) {
    System.out.println("num=" + val);
  }

  private void debugLabel(String label) {
    System.out.println("lab=" + label);
  }

}
