/* Processor.java */

package whitespace;

import java.io.*;
import java.util.*; // ListƂ

import whitespace.Lexer;
import whitespace.Parser;
import whitespace.InstructionNode;
import whitespace.AsmCodeWriter;

public class Processor {

  private static final int LIST = 1;
  private static final int EXEC = 2;

  public static void main(String[] args) {

    int process = EXEC;
    boolean altSyntax = false;
    boolean step = false;

    int argIndex = 0;
    while (argIndex < args.length) {
      if (args[argIndex].equals("-list")) {
	process = LIST;
      } else if (args[argIndex].equals("-exec")) {
	process = EXEC;
      } else if (args[argIndex].equals("-alt")) {
	altSyntax = true;
      } else if (args[argIndex].equals("-log")) {
	step = true;
      } else if (args[argIndex].equals("-help")) {
	System.out.println("Options");
	System.out.println("  -help : \.");
	System.out.println("  -list : AZu(?)̃Xg\.");
	System.out.println("  -exec : siftHg̓j.");
	System.out.println("  -log  : whitespace.logɃOo.");
	System.out.println("  -alt  : ֍\g.");
	System.exit(0);
      } else {
	break;
      }
      argIndex++;
    }

    /* \[X͌ */
    InputStream source = null;
    String sourceName = null;
    boolean mustClose = true;
    if (args.length == argIndex) {
      source = System.in;
      sourceName = "-(stdin)-";
      mustClose = false;
    } else {
      try {
	File file = new File(args[argIndex]);
	source = new FileInputStream(file);
	sourceName = args[argIndex];
	mustClose = true;
      } catch (IOException e) {
	System.err.println(e);
	System.exit(1);
      }
    }
//    System.err.println(argIndex + " " + sourceName);

    /* lexer ZbgAbv */
    WhitespaceChars chars;
    if (altSyntax) {
      chars = new WhitespaceChars('s', 't', 'n', 'r', '0');
    } else {
      chars = null;
    }
    Lexer lexer = new Lexer(source, sourceName, chars);

    /* parser ZbgAbv */
    Parser parser = new Parser(lexer);
    List<InstructionNode> program = null;
    try {
      program = parser.parse();
    } catch (Exception e) {
      System.err.println(e);
      System.exit(1);
    } 

/* // 
      try {
	if (mustClose) {
	  source.close();
	} catch (Exception e) {
	  // uȂ
	}
    }
*/

    /* process */
    if (process == LIST) {
      AsmCodeWriter writer = new AsmCodeWriter(program, System.out);
      try {
	writer.writeCode();
      } catch (IOException e) {
	System.err.println(e);
	System.exit(1);
      }
    } else if (process == EXEC) {
      Interp interp = new Interp(program, new StdIOEnvironment());
      try {
	if (step) {
	  interp.setInspector(new LogInspector());
	}
	interp.setStepping(step);
	interp.execute();
      } catch (Exception e) {
	System.err.println(e);
	System.exit(1);
      } finally {
	// IOEnvironment̏Kv
      }
    } else {
      assert false : "Unsupported processing";
    }
    /* OK, finished */
    System.exit(0);
  }
}
